package main

import (
	"net/http"
	"sprechstundentool/controllers"
	"sprechstundentool/repositories"
	"sprechstundentool/sqldb"
)

func main() {
	db := sqldb.ConnectDB()

	// Create repos
	roomRepo := repositories.NewRoomRepo(db)
	courseRepo := repositories.NewCourseRepo(db)
	tutorRepo := repositories.NewTutorRepo(db)
	officeHourRepo := repositories.NewOfficeHourRepo(db, roomRepo, tutorRepo, courseRepo)

	h := controllers.NewBaseHandler(roomRepo, officeHourRepo, courseRepo, tutorRepo)

	http.HandleFunc("/getByRoom", h.GetByRoomHandler)
	http.HandleFunc("/getByCourse", h.GetByCourseHandler)

	http.HandleFunc("/", h.RootHandler)

	http.ListenAndServe(":8080", nil)
}
